@@ -962,7 +962,7 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(uint idx,
   SplM_plan_info *spl_plan= 0;
   uint best_key= 0;
   uint best_key_parts= 0;
-  table_map best_param_tables;
+  table_map best_param_tables= 0L;
   Json_writer_object trace_obj(thd, "choose_best_splitting");
   Json_writer_array  trace_arr(thd, "considered_keys");
   /*
@@ -1128,14 +1128,17 @@ SplM_plan_info * JOIN_TAB::choose_best_splitting(uint idx,
 
     if (spl_plan)
     {
-      trace_obj.
-        add("lead_table", spl_plan->table->alias.ptr()).
-        add("index",      spl_plan->table->key_info[spl_plan->key].name).
-        add("parts",      spl_plan->parts).
-        add("split_sel",  spl_plan->split_sel).
-        add("cost",       spl_plan->cost).
-        add("records",    (ha_rows) (records * spl_plan->split_sel)).
-        add("refills",    refills);
+      if (unlikely(thd->trace_started()))
+      {
+        trace_obj.
+          add("lead_table", spl_plan->table->alias.ptr()).
+          add("index",      spl_plan->table->key_info[spl_plan->key].name).
+          add("parts",      spl_plan->parts).
+          add("split_sel",  spl_plan->split_sel).
+          add("cost",       spl_plan->cost).
+          add("unsplit_cost", spl_opt_info->unsplit_cost).
+          add("records",    (ha_rows) (records * spl_plan->split_sel));
+      }
 
       if (refills * spl_plan->cost < spl_opt_info->unsplit_cost - 0.01)
       {
