@@ -61,7 +61,7 @@ Relay_log_info::Relay_log_info(bool is_slave_recovery, const char* thread_name)
    gtid_skip_flag(GTID_SKIP_NOT), inited(0), abort_slave(0), stop_for_until(0),
    slave_running(MYSQL_SLAVE_NOT_RUN), until_condition(UNTIL_NONE),
    until_log_pos(0), retried_trans(0), executed_entries(0),
-   sql_delay(0), sql_delay_end(0),
+   last_trans_retry_count(0), sql_delay(0), sql_delay_end(0),
    until_relay_log_names_defer(false),
    m_flags(0)
 {
@@ -87,6 +87,7 @@ Relay_log_info::Relay_log_info(bool is_slave_recovery, const char* thread_name)
   max_relay_log_size= global_system_variables.max_relay_log_size;
   bzero((char*) &info_file, sizeof(info_file));
   bzero((char*) &cache_buf, sizeof(cache_buf));
+  bzero(&last_seen_gtid, sizeof(last_seen_gtid));
   mysql_mutex_init(key_relay_log_info_run_lock, &run_lock, MY_MUTEX_INIT_FAST);
   mysql_mutex_init(key_relay_log_info_data_lock,
                    &data_lock, MY_MUTEX_INIT_FAST);
@@ -1710,7 +1711,8 @@ scan_all_gtid_slave_pos_table(THD *thd, int (*cb)(THD *, LEX_CSTRING *, void *),
   else
   {
     size_t i;
-    Dynamic_array<LEX_CSTRING*> files(dirp->number_of_files);
+    Dynamic_array<LEX_CSTRING*> files(PSI_INSTRUMENT_MEM,
+                                      dirp->number_of_files);
     Discovered_table_list tl(thd, &files);
     int err;
 
@@ -1930,7 +1932,7 @@ rpl_load_gtid_slave_state(THD *thd)
   for (i= 0; i < array.elements; ++i)
   {
     get_dynamic(&array, (uchar *)&tmp_entry, i);
-    if ((err= rpl_global_gtid_slave_state->update(tmp_entry.gtid.domain_id,
+    if ((err= rpl_global_gtid_slave_state->update_nolock(tmp_entry.gtid.domain_id,
                                                   tmp_entry.gtid.server_id,
                                                   tmp_entry.sub_id,
                                                   tmp_entry.gtid.seq_no,
